;;; -*- mode: common-lisp; package: finder; base:10. -*-

;;;****************************************************************

(in-package :finder)

;;;****************************************************************
;;;****************************************************************

;;;this rubber-bands a box with the mouse. we'll use it to select a group of 
;;;file/dir icons.

;;;****************************************************************
;;;****************************************************************

(defun rubber-band-box (win sx sy)
  (let ((start-x sx) (start-y sy)
	(prev-x nil) (prev-y nil))
    ;; Draw a box from the start point to (x, y), erasing
    ;; the previous box (if any)
    (flet ((draw-box (x y)
	     (when start-x
	       ;; erase old box
	       (when prev-x
		 (clim:draw-rectangle* win start-x start-y prev-x prev-y
				  :filled nil
				  :ink clim:+flipping-ink+))
	       ;; draw new
	       (clim:draw-rectangle* win start-x start-y x y
				:filled nil
				:ink clim:+flipping-ink+)
	       ;; remember newly old box
	       (setq prev-x x prev-y y))))
      ;; Doesn't do anything until a button is pressed, then
      ;; rubber bands until the button is released.
      (clim:with-output-recording-options (win :record-p nil)
	(clim:tracking-pointer (win)
	  (:pointer-motion (x y)
	    (draw-box x y))
	  (:presentation (x y)
	    (draw-box x y))
	  (:pointer-button-press (x y)
	    (setq start-x x start-y y))
	  (:pointer-button-release (x y)
	    (return-from rubber-band-box (list (min start-x x) (min start-y y)
					       (max start-x x) (max start-y y)
					      )))
	  ))
      ))
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
;;;
;;; end of file
;;;
;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
