;;; -*- mode: common-lisp; package: finder; base:10. -*-

(in-package :finder)

;;;****************************************************************
;;;****************************************************************

;;;this file contains a definition for the Finder Frame and the menu
;;;comnmands which appear in its command table and in the command-pane
;;;across the top. other commands which are not visible are contained in
;;;other files, edit-objs, other-commands, presentation-translators...

(defvar *finder-frame* ())

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

;;;define the application frame for Finder.

(clim:define-command-table finder-command-table)

;;;cause the ACCEPTING-VALUES windows to inherit all the commands I define for CDP.
(clim:define-command-table clim:accept-values :inherit-from (finder-command-table))

;;;****************************************************************
;;;****************************************************************

(clim:define-application-frame finder-frame (clim:application-frame)
  ((default-dir :initform *default-pathname-defaults* :accessor default-dir)
      ;;
      ;;an important thing to note: selected files contains 
      ;;((string presentation) (string presentation) ...) as its contents.
      ;;this is for quickie purposes of keeping the file-name and it's presentation
      ;;together. [i have since learned of a better way, but am disinclined to fix here]
      ;;
   (selected-files :initform () :accessor selected-files)
      )
  (:panes ((title1 :title
		   :display-string "   File/Directory Manager   "
		   :stream-background clim:+foreground+
		   :stream-foreground clim:+background+)
	   (commands :command-menu
		     :display-after-commands t)
	   (file-display  :application :scroll-bars :both 
			:end-of-page-action :allow
			:end-of-line-action :allow
			:text-margin 5)
	   (buttons     :application :scroll-bars :none
			:incremental-redisplay t
			:display-function 'show-dir-chain)
	   (mouse-doc   :pointer-documentation)))
  (:layout ((default-config (:column 1
				     (title1 :compute)
				     (commands :compute)
				     (file-display :rest)
				     (buttons     :compute)
				     (mouse-doc :compute)))
	    ))
  (:command-table finder-command-table)
  (:command-definer define-finder-command)
  )

;;;****************************************************************

;;;make sure that most commands are off at the beginning.

(defmethod initialize-instance :after ((Myself finder-frame) &rest ignore)
  (declare (ignore ignore))
  
  (frame-bitmap-cursor Myself "~/Lisp-code/dir-mgr/big-arrow.bm")
  
  ;;flip some commands on and off.  
  (dolist (cmd '())
    (clim:disable-command cmd Myself))
  (dolist (cmd '())
    (clim:enable-command cmd Myself))
;  (clim:redisplay-frame-pane myself 'rfc-commands :force-p t)
  )

;;;****************************************************************

;;i don't know what this does, but the listener had it...
(defmethod frame-maintain-presentation-histories ((frame finder-frame))
  t)

;;;****************************************************************
;;;****************************************************************

(defmethod clim::run-frame-top-level :before ((frame finder-frame))
;  (setq *package* (find-package :planner))
  )

;;;****************************************************************
;;;****************************************************************

(defmethod clear-all-panes ((myself finder-frame))
  (dolist (pane '(file-display buttons))
    (clim:window-clear (clim:get-frame-pane myself pane))
    ))

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

;;;commands

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-files :menu ("Files"
					 :documentation 
					 "Pull-down menu for File/Directory handling")) 
    ()
  
  (let ((display (clim:get-frame-pane *finder-frame* 'file-display))
	choice)
    (setq choice (clim:menu-choose '("New..." "Open..." "Close" "Rename..." " "
				     "Print Directory" "Print File" " " "Copy...")
				   :associated-window display
				   ))
    (cond ((string-equal choice "New...")   (com-make-dir))
	  ((string-equal choice "Open...")  (com-open-dir))
	  ((string-equal choice "Close")     (com-close-dir))
	  ((string-equal choice "Rename...") (com-rename-file))
	  ((string-equal choice "Copy...")   (com-copy-files-to-other-dir))
	  ((string-equal choice "Print Directory")     (com-print-dir))
	  ((string-equal choice "Print File")     (com-print-file))
	  )
    ))

;;;****************************************************************

(define-finder-command (com-copy-files-to-other-dir :menu ()) ()
  )

;;;****************************************************************

(define-finder-command (com-rename-file :menu ()) ()
  ;;rename a single file, the most recently selected one.
  (let ((window *query-io*)
	(temp1 (caar (selected-files *finder-frame*))))
    (clim:accepting-values (window :own-window t 
				   :label "Please Enter a New name for the file"
				   :exit-boxes '((:exit  "Click here to use this name")
						 (:abort "Click here to abort")))
		   (clim:with-text-face (:bold window)
		     (format window "~%Please Enter a name for the new file~2%"))
		   (princ (namestring (default-dir clim:*application-frame*)) window)
		   (terpri window) (terpri window)
		   (setq temp1 (clim:accept 'string :Prompt "Directory Name"
					    :default temp1))
		   (terpri window) (terpri window) ;for beauty.
		   (princ "__________________________" window)
		   (terpri window)
		   )
    (excl:run-shell-command 
     (setq clint (concatenate 'string
		   "mv " 
		   (namestring (default-dir clim:*application-frame*))
		   (caar (selected-files *finder-frame*)) 
		   " "
		   (namestring (default-dir clim:*application-frame*))
		   temp1)))
    (com-open-dir)
    )
  )

;;;****************************************************************

(define-finder-command (com-make-dir :menu ()) ()
  ;;create a subdir for the current default directory.
  (let ((window *query-io*)
	result
	temp1)

    (clim:accepting-values (window :own-window t 
				   :label "Please Enter a name for the new sub-directory"
				   :exit-boxes '((:exit  "Click here to use this name")
						 (:abort "Click here to abort")))
		   (clim:with-text-face (:bold window)
		     (format window "~%Please Enter a name for the new sub-directory.~2%"))
		   (princ (namestring (default-dir clim:*application-frame*)) window)
		   (terpri window) (terpri window)
		   (setq temp1 (clim:accept 'string :Prompt "Directory Name"
					    :default (string (gentemp))))
		   (terpri window) (terpri window) ;for beauty.
		   (princ "__________________________" window)
		   (terpri window)
		   )
    ;;create a dir.
    (setq result
      (excl:run-shell-command
       (concatenate 'string
	 "mkdir " (namestring (default-dir clim:*application-frame*)) temp1)))
    (cond ((zerop result) t)		;it worked
	  ((= 1   result) ())		;failed. why?
	  )
    )
  (com-open-dir)
  )

;;;****************************************************************

(define-finder-command (com-open-dir :menu ()) ()
  ;;open and display the selected subdir
  
  (setf (selected-files clim:*application-frame*) ())
  (let ((files (directory (namestring (default-dir *finder-frame*))))
	(display (clim:get-frame-pane *finder-frame* 'file-display))
	(buttons (clim:get-frame-pane *finder-frame* 'buttons))
;	(y-pos 0)
	)
    (clim:window-clear display)
    (clim:window-clear buttons)

    (setq files (sort (mapcar #'(lambda (file)
				  (cond ((member (string-upcase (pathname-type file))
						 '("LISP" "CL") :test #'string-equal)
					 (make-instance 'file-icon
					   :picture lisp-file-icon
					   :name (file-namestring file)
					   ))
					;;this appears to be a working test for "directory-p"
					((probe-file (concatenate 'string
						      (namestring file)
						      "/."))
					 (make-instance 'dir-icon
					   :picture folder-icon
					   :name (file-namestring file)
					   ))
					(t (make-instance 'file-icon
					     :picture file-icon   ;stop-sign-icon
					     :name (file-namestring file)
					     ))))
			      files)
		      #'string-lessp :key #'name))

    (clim:draw-line* display 0 50 1000 50)

    (show-dir-chain clim:*application-frame* display)
    (terpri display)
    (terpri display)

    (ignore-errors			;seems to be necessary. divide-by-zero was occurring.
      (clim:formatting-table (display :multiple-columns t
				    :inter-column-spacing 20
				    :multiple-columns-inter-column-spacing 20
				    )
      (do ((files1 files (nthcdr 4 files1)))
	  ((null files1))
        (clim:formatting-row (display )
	   (clim:formatting-cell (display :align-x :center)
		(pres-1-box (first files1) display))
	   (if (second files1)
	       (clim:formatting-cell (display :align-x :center)
		     (pres-1-box (second files1) display))
	     (clim:formatting-cell (display :align-x :center) display
				   ))
	   (if (third files1)
	       (clim:formatting-cell (display :align-x :center)
		     (pres-1-box (third files1) display))
	     (clim:formatting-cell (display :align-x :center) display
				   ))
	   (if (fourth files1)
	       (clim:formatting-cell (display :align-x :center)
		     (pres-1-box (fourth files1) display))
	     (clim:formatting-cell (display :align-x :center) display
				   ))))
      ;;a dummy row, to keep highlighting happy..?!
      (clim:formatting-row (display )
	   (clim:formatting-cell (display)
		display))
      ))
    (clim:frame-replay *finder-frame* display)
    )
  )

;;;****************************************************************

(defun pres-1-box (obj STR)
  (clim:present obj (type-of obj) :stream str :single-box t)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-close-dir :menu ()) ()
  ;;close this dir and pop up to its parent.
;  (clim:redisplay-frame-pane clim:*application-frame* 'rfc-commands :force-p t)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-print-dir :menu ()) ()
  ;;print the current dir.
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-print-file :menu ()) ()
  ;;print the selected files. skip directories.
  (when (selected-files *finder-frame*)
    (excl:chdir (namestring (default-dir *finder-frame*)))
    (dolist (file (selected-files *finder-frame*))
      (unless (probe-file (concatenate 'string
			    (namestring (car file))
			    "/."))
	(excl:run-shell-command 
	 (concatenate 'string
	   "lpr " 
	   (namestring (default-dir *finder-frame*))
	   (caar (selected-files *finder-frame*))
	   )))
      )
    )
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-edit :menu ("Edit"
					 :documentation 
					 "Pull-down menu for `Edit` handling")) 
    ()
  
  (let ((display (clim:get-frame-pane *finder-frame* 'file-display))
	choice)
    (setq choice (clim:menu-choose '("Cut" "Copy" "Paste" "Select All")
				   :associated-window display
				   ))
    (cond ((string-equal choice "New")   (com-make-dir))
	  ((string-equal choice "Open")  (com-open-dir))
	  ((string-equal choice "Close") (com-close-dir))
	  ((string-equal choice "Print") (com-print-dir))
	  ((string-equal choice "Select All") (com-select-all *finder-frame* display))
	  )
    ))

;;;****************************************************************

(define-finder-command (com-select-all :menu ()) ((frame t) (window t))
  ;;print the selected files. skip directories.
  (excl:chdir (namestring (default-dir *finder-frame*)))
  (dolist (file (directory (namestring (default-dir *finder-frame*))))
    ;;redraw them with highlighting.
    ;;involves com-open-dir somehow.
    )
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-view :menu ("View"
					 :documentation 
					 "Pull-down menu for `View` handling")) 
    ()
  
  (let ((display (clim:get-frame-pane *finder-frame* 'file-display))
	choice)
    (setq choice (clim:menu-choose '("Icon" "Small Icon" "Name" "Date" "Size" "Type")
				   :associated-window display
				   ))
    (cond ((string-equal choice "New")   (com-make-dir))
	  ((string-equal choice "Open")  (com-open-dir))
	  ((string-equal choice "Close") (com-close-dir))
	  ((string-equal choice "Print") (com-print-dir)))
    ))

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-special :menu ("Special"
					 :documentation 
					 "Pull-down menu for `Special` handling")) 
    ()
  
  (let ((display (clim:get-frame-pane *finder-frame* 'file-display))
	choice)
    (setq choice (clim:menu-choose '("Compress File"
				     "UN-Compress File"
				     " "
				     "UU-ENcode File"
				     "UU-DEcode File"
				     " "
				     "TAR Files/Dirs"
				     "View Tar File"
				     "UNTAR File")
				   :associated-window display
				   ))
    (cond ;((string-equal choice "Empty Trash")     (com-empty-trash))
	  ((string-equal choice "Compress File")    (com-compress-file))
	  ((string-equal choice "UN-Compress File") (com-uncompress-file))	  
	  ((string-equal choice "UU-ENcode File")   (com-uuencode-file))
	  ((string-equal choice "UU-DEcode File")   (com-uudecode-file))
	  ((string-equal choice "TAR Files/Dirs")   (com-tar-files))
	  ((string-equal choice "View Tar File")    (com-view-tar-file))
	  ((string-equal choice "UNTAR File")       (com-untar-file))
	  )
    ))

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-tar-files :menu ()) ()
  ;;tar the selected files. skip directories.
  (when (selected-files *finder-frame*)
    (let ((window *query-io*)
	  (temp1 (concatenate 'string
		   (caar (selected-files *finder-frame*))
		   ".tar"))
	  temp2)
      (excl:chdir (namestring (default-dir *finder-frame*)))
      (clim:accepting-values (window :own-window t 
				     :label "Please Enter a New name for the file"
				     :exit-boxes '((:exit  "Click here to use this name")
						   (:abort "Click here to abort")))
	     (clim:with-text-face (:bold window)
	       (format window "~%Please Enter a name for the new file~2%"))
	     (princ (namestring (default-dir *finder-frame*)) window)
	     (terpri window) (terpri window)
	     (setq temp1 (clim:accept 'string :Prompt "Directory Name"
				      :default temp1))
	     (terpri window) (terpri window) ;for beauty.
	     (princ "__________________________" window)
	     (terpri window)
	     )
      (excl:run-shell-command 
       (Apply #'concatenate 'string
	      (append (list 
		       "tar cf " 
		       (namestring (default-dir *finder-frame*))
		       temp1
		       " ")
		      (dolist (fi (selected-files *finder-frame*) temp2)
			(push " " temp2)
			(push (car fi) temp2))))
       )
      (com-open-dir)
      )
    )
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-untar-file :menu ()) ()
  ;;compress the selected files. skip directories.
  (when (selected-files *finder-frame*)
    (excl:chdir (namestring (default-dir *finder-frame*)))
    (excl:run-shell-command 
       (concatenate 'string
	 "tar xf " 
	 (namestring (default-dir *finder-frame*))
	 (caar (selected-files *finder-frame*))
	 ))
    )
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-view-tar-file :menu ()) ()
  (when (selected-files *finder-frame*)
    (excl:run-shell-command 
       (concatenate 'string
	 "tar tf " 
	 (namestring (default-dir *finder-frame*))
	 (caar (selected-files *finder-frame*))
	 ))
    )
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-UUencode-file :menu ()) ()
  ;;UUencode the selected files. skip directories.
  (excl:chdir (namestring (default-dir *finder-frame*)))
  (dolist (file (selected-files *finder-frame*))
    (unless (probe-file (concatenate 'string
			  (namestring (car file))
			  "/."))
      (excl:run-shell-command 
       (concatenate 'string 
	 "uuencode "
	 (namestring (default-dir *finder-frame*))
	 (string (car file))
	 " "
	 (string (car file))
	 )))
    )
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-uudecode-file :menu ()) ()
  ;;uudecode the selected files. skip directories.
  (excl:run-shell-command 
       (concatenate 'string 
	 "cd "
	 (namestring (default-dir *finder-frame*))
	 ))
  (dolist (file (selected-files *finder-frame*))
    (unless (probe-file (concatenate 'string
			  (namestring (car file))
			  "/."))
      (excl:run-shell-command 
       (concatenate 'string 
	 "uudecode "
	 (namestring (default-dir *finder-frame*))
	 (string (car file))
	 )))
    )
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-compress-file :menu ()) ()
  ;;compress the selected files. skip directories.
  (dolist (file (selected-files *finder-frame*))
    (unless (probe-file (concatenate 'string
			  (namestring (car file))
			  "/."))
      (excl:run-shell-command 
       (concatenate 'string 
	 "compress "
	 (namestring (default-dir *finder-frame*))
	 (string (car file)))))
    )
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-goto-homedir :menu ()) ()
  ;;go to the user's homedir. easier than traversing the dir-tree, 
  ;;and common enough to be valuable as a separate thing.
  #+excl (setf (default-dir *finder-frame*) (user-homedir-pathname))
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-uncompress-file :menu ()) ()
  ;;compress the selected files. skip directories.
  (dolist (file (selected-files *finder-frame*))
    (when (search ".Z" (string (car file)))
      (excl:run-shell-command 
       (concatenate 'string 
	 "uncompress "
	 (namestring (default-dir *finder-frame*))
	 (string (car file)))))
    )
  (com-open-dir)
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

(defun run-finder (&optional (root ()))
  
  (when (typep *finder-frame* 'finder-frame)
    (cond ((y-or-n-p "~%There is an existing FINDER window.~2%Should I get rid of it? "))
	  (t (return-from run-finder)))
    
    ;;flip some commands on and off.  
;    (dolist (cmd '())
;      (clim:disable-command cmd *finder-frame*))
;    (dolist (cmd '())
;      (clim:enable-command cmd *finder-frame*))
;
;    (clim:redisplay-frame-pane *finder-frame* 'rfc-commands :force-p t)

    (if (find "FINDER" mp:*all-processes* :test #'string-equal :key #'mp:process-name)
	(mp:process-kill (find "FINDER" mp:*all-processes* :test #'string-equal :key #'mp:process-name)))
    (setq *finder-frame* nil))
  
  (let ((xx (clim:make-application-frame 'finder-frame
					      :parent (OR root (clim:open-root-window :clx))
					      :left 0 :top 50
					      :right 1050 :bottom 800)
	  )
	xwin)

    (setq xwin (slot-value (clim:frame-top-level-window xx) 'clim::window))
    #+XLIB
    (xlib:set-wm-class xwin "clim" "clim")
    (mp:process-run-restartable-function "FINDER" 'clim:run-frame-top-level xx)
    
    (setq *FINDER-frame* xx)
    )
  )

;;;****************************************************************

;;;this fn draw the chain of directory levels as folders. it also draws the "Trash" icon.

(defun show-dir-chain (frame window)
  (let ((files (remove :root (cdr (pathname-directory (default-dir frame))))))

    (setq files (cons (make-instance 'dir-icon
			      :picture folder-icon
			      :name "/"
			      )
		      (mapcar #'(lambda (file)
				  (make-instance 'dir-icon
				    :picture folder-icon
				    :name (file-namestring file)
				    ))
			      files)))
    (clim:formatting-table (window :inter-column-spacing 10
				   :inter-row-spacing 5
				   :multiple-columns t)
       (clim:formatting-row (window)
	 (clim:formatting-cell (window) window))
       (clim:formatting-row (window)
	  (clim:formatting-cell (window) window)
	  (dolist (d files)
	     (clim:formatting-cell (window :align-x :center)
		   (clim:present d 'picture-button :stream window :single-box t)
		   ))
	  (clim:formatting-cell (window)
				(clim:draw-rectangle* window 0 0 100 10
						      :filled nil
						      :ink clim:+background+))
	  
	  ;;also show the TRASH icon.
	  (clim:formatting-cell (window :align-x :center)
	     (clim:present trash 'trash-icon :stream window :single-box t)
	     )
	  ;;show the PRINT icon.
	  (clim:formatting-cell (window :align-x :center)
	     (clim:present print 'print-icon :stream window :single-box t)
	     )
	  ;;show the HOME-DIR icon.
	  (clim:formatting-cell (window :align-x :center)
	     (clim:present home
			   'homedir-icon :stream window :single-box t)
	     )
	  )
       (clim:formatting-row (window)
	 (clim:formatting-cell (window) window))
       )
    )
  )


;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
;;;
;;; end of file
;;;
;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
