;;; -*- mode: common-lisp; package: finder; base:10. -*-

(in-package :finder)

;;;****************************************************************
;;;****************************************************************

;;;some sample code for creating and using picture buttons. this works pretty well.

;;;to use, substitute your command-table name, and your define-...-command name

;;;Written by Clint Hyde, October 1992. Works with CLIM 1.1.

;;;****************************************************************
;;;****************************************************************

;;;first, the defclass. NAME could probably be punted.
;;;PICTURE is of course the icon/bitmap. X and Y are the position in the window.
;;;FORM is a s-expression which gets EVAL'd when the button is clicked.

;;;****************************************************************
;;;****************************************************************

(defclass picture-button ()
	  ((picture :initform () :initarg :picture :accessor picture)
	   (name :initform (INTERN (STRING (GENSYM "PICTURE-BUTTON-"))
				   *package*)
		 :initarg :name :accessor name)
	   (highlighted :initform () :initarg :highlighted :accessor highlighted)
	   (form :initform () :initarg :form :accessor form)
	   (x :initform () :initarg :x :accessor x)
	   (y :initform () :initarg :y :accessor y))
  )

;;;****************************************************************

(defmethod print-object ((myself picture-button) stream)
  (princ "#<" stream)
  (princ (string (type-of myself)) stream)
  (princ " " Stream)
  (princ (name myself) stream)
  (princ ">" stream)
  )

;;;****************************************************************

(clim:define-presentation-method clim:present
				 (myself (p-type picture-button) stream view
					 &key acceptably context-type)
  (declare (ignore acceptably context-type view))
  (clim:with-output-as-presentation (:object myself :stream stream :single-box t)
    (clim:draw-icon* stream (picture-bitmap (picture myself)) 0 0)
    (clim:draw-text* stream (name myself) 0 40 :align-x :center :align-y :bottom)
    )
  )

;;;****************************************************************

;;;this may or may not be a good idea. it probably would get confused with being clicked.

;(clim:define-presentation-method clim:highlight-presentation ((type picture-button) record stream state)
;  (declare (ignore state))	;we'll just use XOR
;  (multiple-value-bind (xoff yoff)
;      (clim:convert-from-relative-to-absolute-coordinates
;       stream (clim:output-record-parent record))
;    (clim:with-bounding-rectangle* (left top right bottom) record
;	   (clim:draw-rectangle* stream
;				 ;;draw the box half-full. was (+ top yoff).
;				 (+ left xoff)  (+ top yoff)
;				 (+ right xoff) (+ bottom yoff)
;				 :ink clim:+flipping-ink+ :filled t)
;	   )
;    )
;  )

;;;****************************************************************
;;;****************************************************************

;;;a macro, for pretty usage.

;(defmacro with-button-blinked (myself stream &body body)
;  stream
;  `(progn
;     (setf (highlighted myself) t)
;     (clim:present ,myself 'ok-button :stream stream :single-box t)
;     (clim:stream-force-output stream)
;     (unwind-protect
;	 (progn ,@body)
;       
;       (sleep 0.5)			;protected forms.
;       (setf (highlighted myself) ())
;       (clim:present ,myself 'ok-button :stream stream :single-box t)
;       (clim:stream-force-output stream)
;       )
;     )
;  )

;;;****************************************************************
;;;****************************************************************

(defclass file-icon (picture-button) ())

;;;****************************************************************

(defclass dir-icon (picture-button) ())

;;;****************************************************************

(defclass trash-icon (picture-button) ())

;;;****************************************************************

(defclass print-icon (picture-button) ())

;;;****************************************************************

(defclass homedir-icon (picture-button) ())

;;;****************************************************************
;;;****************************************************************

(defun invert-record (record stream)
  (multiple-value-bind (xoff yoff)
      (clim:convert-from-relative-to-absolute-coordinates
       stream (clim:output-record-parent record))
    (clim:with-bounding-rectangle* (left top right bottom) record
	   (clim:draw-rectangle* stream
				 ;;draw the box half-full. was (+ top yoff).
				 (+ left xoff)  (+ top yoff)
				 (+ right xoff) (+ bottom yoff)
				 :ink clim:+flipping-ink+ :filled t)
	   )
    )
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
;;;
;;; end of file
;;;
;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
