;;; -*- mode: common-lisp; package: finder; base:10. -*-

(in-package :finder)

;;;****************************************************************
;;;****************************************************************

;;;some code for mousing file/dir icons. this works pretty well.

;;;Written by Clint Hyde, Feb 1993. Works with CLIM 1.1.

;;;****************************************************************
;;;****************************************************************

(define-finder-command (com-load-file :menu nil) ((myself 'file-icon) (stream t))
  (declare (special stream))
  (excl:chdir (namestring (default-dir *finder-frame*)))
  (load (name myself))
  )

;;;****************************************************************

(clim:define-presentation-to-command-translator load-button (file-icon com-load-file 
								       finder-command-table
								       :gesture :describe
								       :pointer-documentation
								       "Load this file"
								       :menu nil)
  (object window)			;arglist ;(presentation context-type frame event x y)
  (list object window)
  )

;;;****************************************************************
;;;****************************************************************

(clim:define-presentation-to-command-translator box-select-files (clim:blank-area
								  com-box-files
								  finder-command-table
								  :gesture :select
								  :pointer-documentation
								  "Mouse-hold&drag to box-select several files"
								  :menu nil)
  (window x y)			;arglist ;(presentation context-type frame event object x y)
  (list window x y)
  )

;;;****************************************************************

(define-finder-command (com-box-files :menu nil) ((stream t) (x t) (y t))
  (declare (special stream))
  (excl:chdir (namestring (default-dir *finder-frame*)))

  ;;clear things.
  (dolist (fi (selected-files clim:*application-frame*))
    (invert-record (cadr fi) stream))

  ;;wipe the old ones out.
  (setf (selected-files clim:*application-frame* ) ())

  (let ((box (rubber-band-box stream x y))
	region)
    (setq region (apply #'clim:make-bounding-rectangle box))
    ;;safety check
    (unless (car box)
      (return-from com-box-files))
    ;;safety check.
    (unless region
      (return-from com-box-files))
    (window-map-over-presentations stream #'pseudo-sel-file region stream)
    ;;remove rubber-band-box trash leftover pixels.
    (clim:draw-rectangle* stream (first box) (second box) (third box) (fourth box)
			  :filled nil
			  :ink clim:+flipping-ink+))
  (dolist (fi (selected-files clim:*application-frame*))
    ;;now highlight the thing to show it's selected...
    (invert-record (cadr fi) stream))
  )

;;;****************************************************************

(defun pseudo-sel-file (rec args &rest ignore &aux (region (car args)) (stream (cadr args)))
  (declare (ignore ignore))
  (when (clim:region-intersects-region-p 
	 (output-record-region rec stream) ;presentation-absolute-location
	 region)
    ;;do we already have this one in the list?
    (if (not (member (name (clim:presentation-object rec)) (selected-files clim:*application-frame*) 
		     :test #'string-equal
		     :key #'car))
	;;NO, add this one.
	(pushnew (list (name (clim:presentation-object rec)) rec)
		 (selected-files clim:*application-frame*))
      )
    )
  )

;;;****************************************************************

;;;SWM: Try something like this instead:

(defun output-record-region* (record stream)
  (clim:with-bounding-rectangle* (left top right bottom) record
    (multiple-value-bind (xoff yoff)
	(clim::convert-from-relative-to-absolute-coordinates
	  stream (clim:output-record-parent record))
      (clim::translate-positions xoff yoff left top right bottom))
    (values left top right bottom)))

(defun output-record-region (record stream)
  (multiple-value-bind (left top right bottom)
      (output-record-region* record stream)
    (clim:make-bounding-rectangle left top right bottom)))

;;;****************************************************************

(defun window-map-over-presentations (window continuation &optional &rest args &aux region record)
  (setq record (slot-value window 'clim::output-record)
	region (car args))
  (labels ((map-loop (record x-offset y-offset)
	    (clim:with-bounding-rectangle* 
		(left top right bottom)
	      record
	      (incf left x-offset)
	      (incf top  y-offset)
	      (incf bottom x-offset)
	      (incf right  y-offset)
	      (if (typep record 'clim::standard-presentation)
		  (funcall continuation record args))
	      (multiple-value-bind (xoff yoff)
		  (clim:output-record-start-position* record)
		(clim:map-over-output-record-elements-overlapping-region
		 record region #'map-loop
		 (- x-offset) (- y-offset)
		 (+ x-offset xoff) (+ y-offset yoff))))))
    (declare (dynamic-extent #'map-loop))
    (map-loop record 0 0)
    ))

;#'(lambda (record args)
;	(describe (clim:presentation-object record)))

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************

;;;this command accomplishes both selecting and moving files.

(define-finder-command (com-select-file :menu nil)
    ((myself 'picture-button) (stream t) (rec t))

  (clim:tracking-pointer (stream :multiple-window t)

    ;;if the user drags over top of a DIR-ICON obj, then copy all the
    ;;selected files into that directory...

    (:presentation (presentation window x y)
	;;move files to this dir
	window x y

	;;do a normal behavior.
	(when (and (typep (clim:presentation-object presentation) 'dir-icon)
		   (not (eql presentation rec)))
	  (invert-record presentation stream)
	  (clim::stream-force-output stream)
	  (sleep 0.5)

	  (let* ((dir (pathname-directory   ;PARENT DIR
		       (namestring (default-dir clim:*application-frame*))))
		 (dir1 (name (slot-value presentation 'clim::object)))   ;TARGET DIR
		 (pos (position dir1 (cdr dir)                     ;CAR is :ABSOLUTE
				:test #'string-equal))
		 ;;full pathname of TARGET DIR
		 (path (make-pathname :directory
				      (if pos
					  (subseq dir 0 (+ 2 pos))
					(append dir (list dir1))
					)))
		 )
	    (excl:chdir (namestring (default-dir *finder-frame*)))
	    ;;for all selected files or directories
	    (dolist (fi (or (selected-files clim:*application-frame*)
			    (list (list (name myself) rec))))
	      ;;move them
	      (excl:run-shell-command  
	       (concatenate 'string
		 "mv "
;		 (namestring (default-dir clim:*application-frame*))
		 (car fi)
		 " "
		 (namestring path))
              )
	      ;;just erase output records...
	      (clim:erase-output-record (cadr fi) stream))

	    (invert-record presentation stream)
	    (return-from com-select-file))
	  )

	;;do the trash icon
	(when (and (typep (clim:presentation-object presentation) 'trash-icon)
		   (not (eql presentation rec)))
	  (invert-record presentation stream)
	  (clim::stream-force-output stream)
	  (sleep 0.5)

	  ;;for all selected files/dirs...
	  (dolist (fi (or (selected-files clim:*application-frame*)
			  (list (list (name myself)))))
	    ;;do the DIR test
	    (if (probe-file (concatenate 'string
			      (namestring (default-dir clim:*application-frame*))
			      (car fi)
			      "/."))
		;;remove a dir
		(progn
		  (excl:run-shell-command  
		   (concatenate 'string
		     "rmdir "
		     (namestring (default-dir clim:*application-frame*))
		     (car fi)
		     ))
		  ;;re-read this dir. could just erase output records...
		  (invert-record rec stream)
		  (clim:erase-output-record rec stream)
		  )
	      ;;remove a file.
	      (ignore-errors (delete-file 
			      (concatenate 'string
				(namestring (default-dir clim:*application-frame*))
				(car fi)
				))
			     ;;re-read this dir. could just erase output records...
			     (invert-record rec stream)
			     (clim:erase-output-record rec stream)
			     ))
	    )				;because the CDR is the presentation.
	  (setf (selected-files clim:*application-frame*) ())
					    
	  (invert-record presentation stream)
	  (return-from com-select-file))

	;;do the PRINTER icon
	(when (and (typep (clim:presentation-object presentation) 'print-icon)
		   (not (eql presentation rec)))
	  (invert-record presentation stream)
	  (clim::stream-force-output stream)
	  (sleep 0.2)

	  ;;for all selected files/dirs...
	  (pushnew (list (name myself) rec)
		   (selected-files clim:*application-frame*))
	  (com-print-file)
	  
	  (invert-record presentation stream)
	  (return-from com-select-file))
	)

    (:pointer-button-release (event x y)
			     event x y

			     (return-from com-select-file)
			     )
    )
  )

;;;****************************************************************

;;;select a file.

(clim:define-presentation-to-command-translator sel-file (file-icon com-select-file
								    finder-command-table
								    :gesture :select
								    :pointer-documentation
								    "Select this File"
								    :menu nil)
  (object window presentation)	;arglist ;( context-type frame event x y)

  ;;clear things.
  (dolist (fi (selected-files clim:*application-frame*))
    (invert-record (cadr fi) window))

  ;;wipe the old ones out.
  (setf (selected-files clim:*application-frame* ) ())

  ;;add this one.
  (pushnew (list (name object) presentation)
	   (selected-files clim:*application-frame*))
    
  ;;now highlight the thing to show it's selected...
  (invert-record presentation window)

  ;call com-select-file with these args...
  (list object window presentation)
  )

;;;****************************************************************

;;;select a directory/folder.

(clim:define-presentation-to-command-translator sel-dir (dir-icon com-select-file
								  finder-command-table
								  :gesture :select
								  :pointer-documentation
								  "Select this Directory"
								  :menu nil)
  (object window presentation)	;arglist ;( context-type frame event x y)
  
  ;;clear things.
  (dolist (fi (selected-files clim:*application-frame*))
    (invert-record (cadr fi) window))

  ;;wipe the old ones out.
  (setf (selected-files clim:*application-frame* ) ())

  ;;add this one.
  (pushnew (list (name object) presentation)
	   (selected-files clim:*application-frame*))
    
  ;;now highlight the thing to show it's selected...
  (invert-record presentation window)

  (list object window presentation)
  )

;;;****************************************************************

;;;a shortcut for getting to one's home directory.

(clim:define-presentation-to-command-translator sel-home-dir (homedir-icon com-goto-homedir
								  finder-command-table
								  :gesture :select
								  :pointer-documentation
								  "Go To Your HOME Directory"
								  :menu nil)
  ()	;arglist ;(object window presentation context-type frame event x y)
  ()
  )

;;;****************************************************************
;;;****************************************************************

;;;select/deselect a file in addition to any already selected.

(define-finder-command (com-also-select-file :menu nil)
    ((myself 'picture-button) (stream t) (record t))
  (declare (special stream))

  ;;do we already have this one in the list?
  (if (not (member (name myself) (selected-files clim:*application-frame*) 
		   :test #'string-equal
		   :key #'car))
      ;;NO, add this one.
      (pushnew (list (name myself) record) (selected-files clim:*application-frame*))
    ;;YES, so remove it.
    (setf (selected-files clim:*application-frame*) 
      (remove (name myself) (selected-files clim:*application-frame*)
	      :test #'string-equal
	      :key #'car)))
  ;;now highlight the thing to show it's selected...
  (invert-record record stream)
  ;;call this one to handle the mouse...
  (com-select-file myself stream record)
  )

;;;****************************************************************

;;;add/remove a file.

(clim:define-presentation-to-command-translator also-sel-file (file-icon com-also-select-file 
									 finder-command-table
									 :gesture :shift-left
									 :pointer-documentation
									 "Also select this file"
									 :menu nil)
  (object window presentation)		;arglist ;( context-type frame event x y)

  (list object window presentation)
  )

;;;****************************************************************

;;;add/remove a directory.

(clim:define-presentation-to-command-translator also-sel-dir (dir-icon com-also-select-file 
								       finder-command-table
								       :gesture :shift-left
								       :pointer-documentation
								       "Also select this Dir"
								       :menu nil)
  (object window presentation)		;arglist ;( context-type frame event x y)
  (list object window presentation)
  )

;;;****************************************************************
;;;****************************************************************

;;;copy selected files/dirs from one place to another.

(define-finder-command (com-copy-files :menu nil)
    ((myself 'picture-button) (stream t) (record t))
  
  (clim:tracking-pointer (stream :multiple-window t
;				 :context-type 'dir-icon
				 )
    ;;if the user drags over top of a DIR-ICON obj, then copy all the
    ;;selected files into that directory...

    (:presentation (presentation window x y)
	;;move files to this dir
	window x y
	(when (and (typep (clim:presentation-object presentation) 'dir-icon)
		   (not (eql presentation record)))
	  (invert-record presentation stream)
	  (clim::stream-force-output stream)
	  (let* ((dir (pathname-directory (namestring (default-dir clim:*application-frame*))))
		 (dir1 (name (slot-value presentation 'clim::object)))
		 (pos (position dir1 (cdr dir)                     ;CAR is :ABSOLUTE
				:test #'string-equal))
		 from-file to-file
		 (path (make-pathname :directory
				      (if pos
					  (subseq dir 0 (+ 2 pos))
					(append dir (list dir1))
					)))
		 )
	    ;;do list of all selected files/dirs.
	    (dolist (fi (or (selected-files clim:*application-frame*)
			    (list (list (name myself)))))
	      ;;where it is and where it goes...
	      (setq from-file (concatenate 'string
				(namestring (default-dir clim:*application-frame*))
				(car fi) ;from
				)
		    to-file (concatenate 'string
				(namestring path)
				(car fi) ;to
				)
		    )
	      ;;if the target file already exists, query whether to continue (replacing target).
	      (when (probe-file to-file)
		(clim:accepting-values (*query-io* :own-window t)
		       (format *query-io* "~2%Copying ~S ~%into ~S~2%" from-file to-file)
		       (cond ((= (file-write-date from-file)
				 (file-write-date to-file))
			      (format *query-io* "~2%The target file already exists")
			      (format *query-io* ", and both have the same date.~2%")
			      (setq to-file (concatenate 'string
					      (namestring path)
					      (clim:accept 'string
							 :STREAM *query-io*
							 :default "NEW"
							 :prompt "New name") ;to
					      )))
			     ((< (file-write-date from-file)
				 (file-write-date to-file))
			      (format *query-io* "~2%The target file already exists")
			      (format *query-io* ", and the target is newer.~%"))
			     ((> (file-write-date from-file)
				 (file-write-date to-file))
			      (format *query-io* "~2%The target file already exists")
			      (format *query-io* ", and the target is older.~%"))
			     (t 
			      (format *query-io* "~2%Copy ~S ~%into ~S ?~2%" from-file to-file))
			     )
		       (terpri *query-io*)(terpri *query-io*)
		       (Princ "____________________" *query-io*))
		
		(when (probe-file to-file)   ;do it again, because it might have changed...
		  (delete-file to-file)))
	      
	      ;;copy from-file into to-file
	      (excl:run-shell-command
	       (concatenate 'string
			     "cp -r "
			     ;;odd...have to use the entire pathname for copy...
			     (namestring (default-dir clim:*application-frame*))
			     (car fi)	;from
			     " "	;required from..to separator
			     (namestring to-file)   ;to
			     )))
					    
	    ;;visual clean-up.
	    (invert-record presentation stream)
	    (return-from com-copy-files))
	  )
	)
    (:pointer-button-release (event x y)
			     event x y
			     ;;clear things.
			     (dolist (fi (selected-files clim:*application-frame*))
			       (invert-record (cadr fi) stream))
			     ;;wipe them out.
			     (setf (selected-files clim:*application-frame* ) ())

			     ;;add this one.
			     (pushnew (list (name myself) record)
				      (selected-files clim:*application-frame*))
			     ;;now highlight the thing to show it's selected...
			     (invert-record record stream)
			     (return-from com-copy-files)
			     )
    )
  )

;;;****************************************************************

(clim:define-presentation-to-command-translator copy-files (file-icon com-copy-files
								      finder-command-table
								      :gesture :control-left
								      :pointer-documentation
								      "Copy the selected files"
								      :menu nil)
  (object window presentation)		;arglist ;(presentation context-type frame event x y)
  (list object window presentation)
  )

;;;****************************************************************

(clim:define-presentation-to-command-translator copy-dirs (dir-icon com-copy-files
								    finder-command-table
								    :gesture :control-left
								    :pointer-documentation
								    "Copy the selected files"
								    :menu nil)
  (object window presentation)		;arglist ;(presentation context-type frame event x y)
  (list object window presentation)
  )

;;;****************************************************************
;;;****************************************************************

(clim:define-presentation-to-command-translator open-dir-icon (dir-icon com-open-dir 
									finder-command-table
									:gesture :middle
									:pointer-documentation
									"Open this Directory"
									:menu nil)
  (object)			;arglist ;( window presentation context-type frame event x y)
  (let* ((dir (remove :root (pathname-directory (namestring (default-dir *finder-frame*)))))
	 (pos (position (name object) (cdr dir) ;CAR is :ABSOLUTE
			:test #'string-equal))
	 )
    (setf (default-dir clim:*application-frame*)
          (if (string-equal (name object) "/")
	      "/"
	      (make-pathname :directory
		     (if pos
			 (subseq dir
				 0
				 (+ 2 pos))
		       (append dir (list (name object)))
		       )))
	  )
;  (list object window)
    )
  ()
  )

;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
;;;
;;; end of file
;;;
;;;****************************************************************
;;;****************************************************************
;;;****************************************************************
